`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Engineer: Magnus Øverbø
// 
// Create Date: 02/12/2019 10:14:58 AM
// Design Name: 
// Module Name: ciphersearch
// Project Name: FPGA Cipher Search
// Target Devices: Nexys A7 100T
// Description: Performs an exhaustive ARBP search on a bitsequence generated by
// 							an LFSR for all its initial states.  When the search is done,
// 							the collected data is sent to the computer via UART / RS232 on
// 							the JTAG port.
//
// Dependencies: uart.v, arbp.v
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:	Source code for UART communication was collected from a
// Git repository under MIT License and modified to fit this project. It was 
// created by Tim Goddard, http://goddard.net.nz.
// Repo: https://github.com/cyrozap/osdvu
// Version: https://github.com/cyrozap/osdvu/blob/d18488c41141cfb1c7b29f5a5840510e727ae5a2/uart.v
// 
//////////////////////////////////////////////////////////////////////////////////
//Definitions of static variables
`define M 8				        	      //Size of cipher/prefix/search word
`define N 16	           				  //Size of search text
`define K 3					        	  //Max error size
`define POL 4					          //Polynomial degree
`define LEDLEN 16
`define Blen 8

module ciphersearch(clk);

	//IO variables
	input 	clk;						    //external clock
	
	//Global variables
	reg        [`POL-1:0] POLYNOMIAL;       //Polynomial
	reg 	   [(`M-1):0] CIPHER;           //Non-linear cipher text
	reg	       [(`M-1):0] B [0:1];          //Prefix set
	
	
	//ARBP search
    reg                   arbpReset;        //Reset
	reg        [`POL-1:0] arbpInitState;    //Initial state of LFSR, 1-FFFFF...Fn
	wire                  arbpTxSignalWire;     //Wire for tx signal
	reg                   arbpTxSignal;
		
	           
	initial begin
		CIPHER			= `M'b11011000;                //Cipher text
		B[0]			= `M'b11111111 ^ CIPHER;       //Calculate prefix of 0
		B[1]			= `M'b11111111 & CIPHER;       //Calculate prefix of 1
		POLYNOMIAL	    = `POL'b1011;
		
		arbpInitState   = `POL'b0001;
	end
	
endmodule

